@@ -8,6 +8,8 @@ test_that("tax_range_time() works", {
 
   # Expect true
   expect_true(is.data.frame(tax_range_time(occdf = occdf)))
+  expect_true(is.data.frame(tax_range_time(occdf = occdf, by = "LAD")))
+  expect_true(is.data.frame(tax_range_time(occdf = occdf, by = "name")))
 
   # Expect equal
   expect_equal(
@@ -24,6 +26,7 @@ test_that("tax_range_time() works", {
   expect_error(tax_range_time(occdf = occdf, by = "test"))
   expect_error(tax_range_time(occdf = occdf, plot = "test"))
   expect_error(tax_range_time(occdf = occdf, name = "test"))
+  expect_error(tax_range_time(occdf = occdf, plot_args = "test"))
   occdf$genus[1] <- NA
   expect_error(tax_range_time(occdf = occdf))
   occdf$max_ma[1] <- "test"
